@@ -7,10 +7,9 @@
 #       Github:     https://github.com/thieunguyen5991                                                  %
 #-------------------------------------------------------------------------------------------------------%
 
-from numpy.random import uniform, randn, random, normal
-from numpy import ones, sin, pi, power, abs, sqrt, sign
+from numpy.random import uniform, randn, random
+from numpy import ones, power
 from copy import deepcopy
-from math import gamma
 from mealpy.root import Root
 
 
@@ -23,8 +22,8 @@ class BaseTWO(Root):
     ID_FIT = 1
     ID_WEIGHT = 2
 
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        Root.__init__(self, root_paras)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        Root.__init__(self, objective_func, problem_size, domain_range, log)
         self.epoch = epoch
         self.pop_size = pop_size
 
@@ -98,14 +97,14 @@ class BaseTWO(Root):
             if current_best[self.ID_FIT] > g_best[self.ID_FIT]:
                 g_best = deepcopy(current_best)
             self.loss_train.append(1.0 / g_best[self.ID_FIT] - self.EPSILON)
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, 1.0 / g_best[self.ID_FIT] - self.EPSILON))
         return g_best[self.ID_POS], 1.0 / g_best[self.ID_FIT] - self.EPSILON, self.loss_train
 
 
 class OppoTWO(BaseTWO):
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BaseTWO.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BaseTWO.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         pop_old = [self._create_solution__(minmax=self.ID_MAX_PROB) for _ in range(self.pop_size)]
@@ -132,9 +131,7 @@ class OppoTWO(BaseTWO):
                 if pop_old[i][self.ID_FIT] < pop_new[i][self.ID_FIT]:
                     pop_old[i] = deepcopy(pop_new[i])
                 else:
-                    t1 = self.domain_range[1] * ones(self.problem_size) + self.domain_range[0] * ones(self.problem_size)
-                    t2 = -1 * g_best[self.ID_POS] + uniform() * (g_best[self.ID_POS] - pop_old[i][self.ID_POS])
-                    C_op = t1 + t2
+                    C_op = self._create_opposition_solution__(pop_old[i][self.ID_POS], g_best[self.ID_POS])
                     fit_op = self._fitness_model__(C_op, self.ID_MAX_PROB)
                     if fit_op > pop_old[i][self.ID_FIT]:
                         pop_old[i] = [C_op, fit_op, 0.0]
@@ -145,14 +142,14 @@ class OppoTWO(BaseTWO):
             if current_best[self.ID_FIT] > g_best[self.ID_FIT]:
                 g_best = deepcopy(current_best)
             self.loss_train.append(1.0 / g_best[self.ID_FIT] - self.EPSILON)
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, 1.0 / g_best[self.ID_FIT] - self.EPSILON))
         return g_best[self.ID_POS], 1.0 / g_best[self.ID_FIT] - self.EPSILON, self.loss_train
 
 
 class OTWO(BaseTWO):
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BaseTWO.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BaseTWO.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         pop_temp = [self._create_solution__(minmax=self.ID_MAX_PROB) for _ in range(self.pop_size)]
@@ -187,9 +184,7 @@ class OTWO(BaseTWO):
                 if pop_old[i][self.ID_FIT] < pop_new[i][self.ID_FIT]:
                     pop_old[i] = deepcopy(pop_new[i])
                 else:
-                    t1 = self.domain_range[1] * ones(self.problem_size) + self.domain_range[0] * ones(self.problem_size)
-                    t2 = -1 * g_best[self.ID_POS] + uniform() * (g_best[self.ID_POS] - pop_old[i][self.ID_POS])
-                    C_op = t1 + t2
+                    C_op = self._create_opposition_solution__(pop_old[i][self.ID_POS], g_best[self.ID_POS])
                     fit_op = self._fitness_model__(C_op, self.ID_MAX_PROB)
                     if fit_op > pop_old[i][self.ID_FIT]:
                         pop_old[i] = [C_op, fit_op, 0.0]
@@ -200,34 +195,14 @@ class OTWO(BaseTWO):
             if current_best[self.ID_FIT] > g_best[self.ID_FIT]:
                 g_best = deepcopy(current_best)
             self.loss_train.append(1.0 / g_best[self.ID_FIT] - self.EPSILON)
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, 1.0 / g_best[self.ID_FIT] - self.EPSILON))
         return g_best[self.ID_POS], 1.0 / g_best[self.ID_FIT] - self.EPSILON, self.loss_train
 
 
 class LevyTWO(BaseTWO):
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BaseTWO.__init__(self, root_paras, epoch, pop_size)
-
-    def _levy_flight__(self, epoch, solution, prey):
-        self.beta = 1
-        # muy and v are two random variables which follow normal distribution
-        # sigma_muy : standard deviation of muy
-        sigma_muy = power(gamma(1 + self.beta) * sin(pi * self.beta / 2) / (gamma((1 + self.beta) / 2) * self.beta * power(2, (self.beta - 1) / 2)), 1 / self.beta)
-        # sigma_v : standard deviation of v
-        sigma_v = 1
-        muy = normal(0, sigma_muy)
-        v = normal(0, sigma_v)
-        s = muy / power(abs(v), 1 / self.beta)
-        # D is a random solution
-        D = self._create_solution__(minmax=self.ID_MAX_PROB)
-        LB = 0.01 * s * (solution[self.ID_POS] - prey[self.ID_POS])
-
-        levy = D[self.ID_POS] * LB
-        return levy
-
-        #x_new = solution[0] + 1.0/sqrt(epoch+1) * sign(uniform() - 0.5) * levy
-        #return x_new
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BaseTWO.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         pop_old = [self._create_solution__(minmax=self.ID_MAX_PROB) for _ in range(self.pop_size)]
@@ -238,7 +213,7 @@ class LevyTWO(BaseTWO):
         for epoch in range(self.epoch):
             for i in range(self.pop_size):
                 if uniform() < 0.5:
-                    pop_new[i][self.ID_POS] = self._levy_flight__(epoch+1, pop_new[i], g_best)
+                    pop_new[i][self.ID_POS] = self._levy_flight__(epoch=epoch, solution=pop_new[i][self.ID_POS], g_best=g_best[self.ID_POS], step=0.001, case=1)
                 else:
                     for j in range( self.pop_size):
                         if pop_old[i][self.ID_WEIGHT] < pop_old[j][self.ID_WEIGHT]:
@@ -263,14 +238,14 @@ class LevyTWO(BaseTWO):
             if current_best[self.ID_FIT] > g_best[self.ID_FIT]:
                 g_best = deepcopy(current_best)
             self.loss_train.append(1.0 / g_best[self.ID_FIT] - self.EPSILON)
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, 1.0 / g_best[self.ID_FIT] - self.EPSILON))
         return g_best[self.ID_POS], 1.0 / g_best[self.ID_FIT] - self.EPSILON, self.loss_train
 
 
 class ITWO(OppoTWO, LevyTWO):
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        OppoTWO.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        OppoTWO.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         pop_old = [self._create_solution__(minmax=self.ID_MAX_PROB) for _ in range(self.pop_size)]
@@ -280,7 +255,7 @@ class ITWO(OppoTWO, LevyTWO):
         for epoch in range(self.epoch):
             for i in range(self.pop_size):
                 if uniform() < 0.5:
-                    pop_new[i][self.ID_POS] = self._levy_flight__(epoch + 1, pop_new[i], g_best)
+                    pop_new[i][self.ID_POS] = self._levy_flight__(epoch=epoch, solution=pop_new[i][self.ID_POS], g_best=g_best[self.ID_POS], step=0.001, case=1)
                 else:
                     for j in range(self.pop_size):
                         if pop_old[i][self.ID_WEIGHT] < pop_old[j][self.ID_WEIGHT]:
@@ -299,8 +274,7 @@ class ITWO(OppoTWO, LevyTWO):
                 if pop_old[i][self.ID_FIT] < pop_new[i][self.ID_FIT]:
                     pop_old[i] = deepcopy(pop_new[i])
                 else:
-                    C_op = self.domain_range[1] * ones(self.problem_size) + self.domain_range[0] * ones(self.problem_size) + \
-                           -1 * g_best[self.ID_POS] + uniform() * (g_best[self.ID_POS] - pop_old[i][self.ID_POS])
+                    C_op = self._create_opposition_solution__(pop_old[i][self.ID_POS], g_best[self.ID_POS])
                     fit_op = self._fitness_model__(C_op, self.ID_MAX_PROB)
                     if fit_op > pop_old[i][self.ID_FIT]:
                         pop_old[i] = [C_op, fit_op, 0.0]
@@ -311,6 +285,6 @@ class ITWO(OppoTWO, LevyTWO):
             if current_best[self.ID_FIT] > g_best[self.ID_FIT]:
                 g_best = deepcopy(current_best)
             self.loss_train.append(1.0 / g_best[self.ID_FIT] - self.EPSILON)
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, 1.0 / g_best[self.ID_FIT] - self.EPSILON))
         return g_best[self.ID_POS], 1.0 / g_best[self.ID_FIT] - self.EPSILON, self.loss_train
