@@ -25,7 +25,7 @@ LPFA: after 4 loop, it reaches value 100.0      (a huge improvement)
 IPFA: after 2 loop, it reaches value 100.0      (best improvement)
 """
 
-from numpy import exp, ones, sqrt, setxor1d, array, pi, power, abs, sin, sign
+from numpy import exp, sqrt, setxor1d, array, pi, power, abs, sin, sign
 from numpy.random import uniform, choice, normal, random
 from numpy.linalg import norm
 from copy import deepcopy
@@ -37,8 +37,8 @@ class BasePFA(Root):
     """
     A new meta-heuristic optimizer: Pathfinder algorithm
     """
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        Root.__init__(self, root_paras)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        Root.__init__(self, objective_func, problem_size, domain_range, log)
         self.epoch = epoch
         self.pop_size = pop_size
 
@@ -85,7 +85,7 @@ class BasePFA(Root):
             ## Update the best solution found so far (current pathfinder)
             pop, gbest_present = self._update_global_best__(pop, self.ID_MIN_PROB, gbest_present)
             self.loss_train.append(gbest_present[self.ID_FIT])
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(i + 1, gbest_present[self.ID_FIT]))
 
         return gbest_present[self.ID_POS], gbest_present[self.ID_FIT], self.loss_train
@@ -93,8 +93,8 @@ class BasePFA(Root):
 
 class OPFA(BasePFA):
 
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BasePFA.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BasePFA.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         # Init pop and calculate fitness
@@ -136,9 +136,7 @@ class OPFA(BasePFA):
                 if fit < pop[j][self.ID_FIT]:
                     pop[j] = [temp2, fit]
                 else:
-                    C_op = self.domain_range[1] * ones(self.problem_size) + self.domain_range[0] * \
-                           ones(self.problem_size) - gbest_present[self.ID_POS] + uniform() * \
-                           (gbest_present[self.ID_POS] - temp2)
+                    C_op = self._create_opposition_solution__(temp2, gbest_present[self.ID_POS])
                     fit_op = self._fitness_model__(C_op)
                     if fit_op < pop[j][self.ID_FIT]:
                         pop[j] = [C_op, fit_op]
@@ -146,34 +144,15 @@ class OPFA(BasePFA):
             ## Update the best solution found so far (current pathfinder)
             gbest_present = self._update_global_best__(pop, self.ID_MIN_PROB, gbest_present)
             self.loss_train.append(gbest_present[self.ID_FIT])
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(i + 1, gbest_present[self.ID_FIT]))
 
         return gbest_present[self.ID_POS], gbest_present[self.ID_FIT], self.loss_train
 
 
 class LPFA(BasePFA):
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BasePFA.__init__(self, root_paras, epoch, pop_size)
-
-    def _levy_flight__(self, epoch, solution, prey):
-        beta = 1
-        # muy and v are two random variables which follow normal distribution
-        # sigma_muy : standard deviation of muy
-        sigma_muy = power(gamma(1 + beta) * sin(pi * beta / 2) / (gamma((1 + beta) / 2) * beta * power(2, (beta - 1) / 2)),1 / beta)
-        # sigma_v : standard deviation of v
-        sigma_v = 1
-        muy = normal(0, sigma_muy)
-        v = normal(0, sigma_v)
-        s = muy / power(abs(v), 1 / beta)
-        # D is a random solution
-        D = self._create_solution__()
-        LB = 0.01 * s * (solution[self.ID_POS] - prey[self.ID_POS])
-
-        levy = D[self.ID_POS] * LB
-        return levy
-        #x_new = solution[self.ID_POS] + 1.0/sqrt(epoch+1) * sign(uniform() - 0.5) * levy
-        #return x_new
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BasePFA.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         # Init pop and calculate fitness
@@ -183,9 +162,9 @@ class LPFA(BasePFA):
         pop, g_best = self._sort_pop_and_get_global_best__(pop, self.ID_FIT, self.ID_MIN_PROB)
         gbest_present = deepcopy(g_best)
 
-        for i in range(self.epoch):
+        for epoch in range(self.epoch):
             alpha, beta = uniform(1, 2, 2)
-            A = uniform(self.domain_range[0], self.domain_range[1]) * exp(-2 * (i + 1) / self.epoch)
+            A = uniform(self.domain_range[0], self.domain_range[1]) * exp(-2 * (epoch + 1) / self.epoch)
 
             ## Update the position of pathfinder and check the bound
             temp = gbest_present[self.ID_POS] + 2 * uniform() * (gbest_present[self.ID_POS] - g_best[self.ID_POS]) + A
@@ -197,40 +176,39 @@ class LPFA(BasePFA):
             pop[0] = deepcopy(gbest_present)
 
             ## Update positions of members, check the bound and calculate new fitness
-            for j in range(1, self.pop_size):
-                temp1 = deepcopy(pop[j][self.ID_POS])
+            for i in range(1, self.pop_size):
+                temp1 = deepcopy(pop[i][self.ID_POS])
                 temp2 = deepcopy(temp1)
                 if uniform() < 0.5:
                     t1 = beta * uniform() * (gbest_present[self.ID_POS] - temp1)
                     for k in range(1, self.pop_size):
                         dist = norm(pop[k][self.ID_POS] - temp1)
                         t2 = alpha * uniform() * (pop[k][self.ID_POS] - temp1)
-                        t3 = uniform(self.domain_range[0], self.domain_range[1], self.problem_size) * (1 - (i + 1) * 1.0 / self.epoch) * dist
+                        t3 = uniform(self.domain_range[0], self.domain_range[1], self.problem_size) * (1 - (epoch + 1) * 1.0 / self.epoch) * dist
                         temp2 += t2 + t3
                     temp2 += t1
                 else:
                     ## Using Levy-flight to boost algorithm's convergence speed
-                    temp2 = self._levy_flight__(i, pop[j], gbest_present)
+                    temp2 = self._levy_flight__(epoch, pop[i][self.ID_POS], gbest_present[self.ID_POS])
 
                 ## Update members
                 temp2 = self._amend_solution_faster__(temp2)
                 fit = self._fitness_model__(temp2)
-                if fit < pop[j][self.ID_FIT]:
-                    pop[j] = [temp2, fit]
+                if fit < pop[i][self.ID_FIT]:
+                    pop[i] = [temp2, fit]
 
             ## Update the best solution found so far (current pathfinder)
             gbest_present = self._update_global_best__(pop, self.ID_MIN_PROB, gbest_present)
             self.loss_train.append(gbest_present[self.ID_FIT])
-            if self.print_train:
-                print("> Epoch: {}, Best fit: {}".format(i + 1, gbest_present[self.ID_FIT]))
+            if self.log:
+                print("> Epoch: {}, Best fit: {}".format(epoch + 1, gbest_present[self.ID_FIT]))
 
         return gbest_present[self.ID_POS], gbest_present[self.ID_FIT], self.loss_train
 
 
 class IPFA(LPFA):
-
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        LPFA.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        LPFA.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         # Init pop and calculate fitness
@@ -240,9 +218,9 @@ class IPFA(LPFA):
         pop, g_best = self._sort_pop_and_get_global_best__(pop, self.ID_FIT, self.ID_MIN_PROB)
         gbest_present = deepcopy(g_best)
 
-        for i in range(self.epoch):
+        for epoch in range(0, self.epoch):
             alpha, beta = uniform(1, 2, 2)
-            A = uniform(self.domain_range[0], self.domain_range[1]) * exp(-2 * (i + 1) / self.epoch)
+            A = uniform(self.domain_range[0], self.domain_range[1]) * exp(-2 * (epoch + 1) / self.epoch)
 
             ## Update the position of pathfinder and check the bound
             temp = gbest_present[self.ID_POS] + 2*uniform()*( gbest_present[self.ID_POS] - g_best[self.ID_POS]) + A
@@ -254,39 +232,37 @@ class IPFA(LPFA):
             pop[0] = deepcopy(gbest_present)
 
             ## Update positions of members, check the bound and calculate new fitness
-            for j in range(1, self.pop_size):
-                temp1 = deepcopy(pop[j][self.ID_POS])
+            for i in range(1, self.pop_size):
+                temp1 = deepcopy(pop[i][self.ID_POS])
                 temp2 = deepcopy(temp1)
                 if uniform() < 0.5:
                     t1 = beta * uniform() * (gbest_present[self.ID_POS] - temp1)
                     for k in range(1, self.pop_size):
                         dist = norm(pop[k][self.ID_POS] - temp1)
                         t2 = alpha * uniform() * (pop[k][self.ID_POS] - temp1)
-                        t3 = uniform(self.domain_range[0], self.domain_range[1], self.problem_size) * (1 - (i + 1) * 1.0 / self.epoch) * dist
+                        t3 = uniform(self.domain_range[0], self.domain_range[1], self.problem_size) * (1 - (epoch + 1) * 1.0 / self.epoch) * dist
                         temp2 += t2 + t3
                     temp2 += t1
                 else:
                     ## Using Levy-flight to boost algorithm's convergence speed
-                    temp2 = self._levy_flight__(i, pop[j], gbest_present)
+                    temp2 = self._levy_flight__(epoch, pop[i][self.ID_POS], gbest_present[self.ID_POS])
 
                 ## Update members based on Opposition-based learning
                 temp2 = self._amend_solution_faster__(temp2)
                 fit = self._fitness_model__(temp2)
-                if fit < pop[j][self.ID_FIT]:
-                    pop[j] = [temp2, fit]
+                if fit < pop[i][self.ID_FIT]:
+                    pop[i] = [temp2, fit]
                 else:
-                    C_op = self.domain_range[1] * ones(self.problem_size) + self.domain_range[0] * \
-                           ones(self.problem_size) - gbest_present[self.ID_POS] + uniform() * \
-                           (gbest_present[self.ID_POS] - temp2)
+                    C_op = self._create_opposition_solution__(temp, gbest_present[self.ID_POS])
                     fit_op = self._fitness_model__(C_op)
-                    if fit_op < pop[j][self.ID_FIT]:
-                        pop[j] = [C_op, fit_op]
+                    if fit_op < pop[i][self.ID_FIT]:
+                        pop[i] = [C_op, fit_op]
 
             ## Update the best solution found so far (current pathfinder)
             gbest_present = self._update_global_best__(pop, self.ID_MIN_PROB, gbest_present)
             self.loss_train.append(gbest_present[self.ID_FIT])
-            if self.print_train:
-                print("> Epoch: {}, Best fit: {}".format(i + 1, gbest_present[self.ID_FIT]))
+            if self.log:
+                print("> Epoch: {}, Best fit: {}".format(epoch + 1, gbest_present[self.ID_FIT]))
 
         return gbest_present[self.ID_POS], gbest_present[self.ID_FIT], self.loss_train
 
@@ -296,8 +272,8 @@ class DePFA(BasePFA):
     """
     A new meta-heuristic optimizer: Pathfinder algorithm
     """
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        BasePFA.__init__(self, root_paras, epoch, pop_size)
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        BasePFA.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _train__(self):
         # Init pop and calculate fitness
@@ -341,7 +317,7 @@ class DePFA(BasePFA):
             ## Update the best solution found so far (current pathfinder)
             pop, gbest_present = self._sort_pop_and_update_global_best__(pop, self.ID_MIN_PROB, gbest_present)
             self.loss_train.append(gbest_present[self.ID_FIT])
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(i + 1, gbest_present[self.ID_FIT]))
 
         return gbest_present[self.ID_POS], gbest_present[self.ID_FIT], self.loss_train
@@ -351,27 +327,8 @@ class LevyDePFA(DePFA):
     """
     A new meta-heuristic optimizer: Pathfinder algorithm
     """
-    def __init__(self, root_paras=None, epoch=750, pop_size=100):
-        DePFA.__init__(self, root_paras, epoch, pop_size)
-
-    def _levy_flight__(self, epoch, solution, prey):
-        beta = 1
-        # muy and v are two random variables which follow normal distribution
-        # sigma_muy : standard deviation of muy
-        sigma_muy = power(gamma(1 + beta) * sin(pi * beta / 2) / (gamma((1 + beta) / 2) * beta * power(2, (beta - 1) / 2)),1 / beta)
-        # sigma_v : standard deviation of v
-        sigma_v = 1
-        muy = normal(0, sigma_muy)
-        v = normal(0, sigma_v)
-        s = muy / power(abs(v), 1 / beta)
-        # D is a random solution
-        D = self._create_solution__()
-        LB = 0.001 * s * (solution[self.ID_POS] - prey[self.ID_POS])
-
-        levy = D[self.ID_POS] * LB
-        return levy
-        #x_new = solution[self.ID_POS] + 1.0/sqrt(epoch+1) * sign(uniform() - 0.5) * levy
-        #return x_new
+    def __init__(self, objective_func=None, problem_size=50, domain_range=(-1, 1), log=True, epoch=750, pop_size=100):
+        DePFA.__init__(self, objective_func, problem_size, domain_range, log, epoch, pop_size)
 
     def _caculate_xichma__(self, beta):
         up = gamma(1 + beta) * sin(pi * beta / 2)
@@ -428,7 +385,7 @@ class LevyDePFA(DePFA):
             ## Update the best solution found so far (current pathfinder)
             pop, g_best = self._sort_pop_and_update_global_best__(pop,self.ID_MIN_PROB, g_best)
             self.loss_train.append(g_best[self.ID_FIT])
-            if self.print_train:
+            if self.log:
                 print("> Epoch: {}, Best fit: {}".format(epoch + 1, g_best[self.ID_FIT]))
 
         return g_best[self.ID_POS], g_best[self.ID_FIT], self.loss_train
